unit DD74u1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

const ver='1.0.0';

type
  TDD74f1 = class(TForm)
    laID: TLabel;
    buLoadMemo: TButton;
    buPlay: TButton;
    buQuit: TButton;
    Memo1: TMemo;
    buSave: TButton;
    laQuestion: TLabel;
    eAns: TEdit;
    laFeedback: TLabel;
    procedure buQuitClick(Sender: TObject);
    procedure buLoadMemoClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure buPlayClick(Sender: TObject);
    procedure eAnsKeyPress(Sender: TObject; var Key: Char);
    procedure buSaveClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
    wMaxQNumber:word;
    (*wMaxQNumber should be 3 if there are 3 questions in the file*)
    bFirstQLine:byte;
    (*wFirstQLine should be 1 if the first line of the file is a question line,
      2 if there is one header line, etc.*)
    bQID,bHistory,bAns:byte;
    (*If the data in the file is in format....
        iihhhhhaaaqqqqqq...
      Where... ii is the question ID,
               hhhhh is the history of the user's success
               aaa is the answer, and
               qqq... (any length) is the question, then set....
         bQID:=2;
         bHistory:=5;
         bAns:=3
      (I.e. these variables tell you how many characters have been assigned
         to each field in the record.*)
    sQID,sHist,sAns,sQ:string;
    wQues:word;
  public
    { Public declarations }
  end;

var
  DD74f1: TDD74f1;

implementation

{$R *.DFM}
{+R$}

procedure TDD74f1.buLoadMemoClick(Sender: TObject);
begin
memo1.Lines.LoadFromFile('DD74questions.txt');
wMaxQNumber:=3;
bFirstQLine:=1;
bQID:=2;
bHistory:=5;
bAns:=2;
buPlay.enabled:=true;
end;

procedure TDD74f1.FormCreate(Sender: TObject);
begin
DD74f1.caption:=DD74f1.caption+', version:'+ver;
randomize;
end;

procedure TDD74f1.buPlayClick(Sender: TObject);
var sRecord:string;

   procedure ParseRecord(sRecord:string;var sQID,sHist,sAns,sQ:string);
   begin
     sQID:=copy(sRecord,1,bQID);
     sHist:=copy(sRecord,bQID+1,bHistory);
     sAns:=copy(sRecord,bQID+bHistory+1,bAns);
     sQ:=copy(sRecord,bQID+bHistory+bAns+1,length(sRecord));
        //The previous line is a little sloppy. It assumes
        //that copy won't return characters from beyond the
        //end of sRecord, which is protection built into
        //copy, I believe.
   end;

begin
wQues:=random(wMaxQNumber)+bFirstQLine-1; (*random(4) returns 0,1,2 or 3... but not 4..
    The "+bFirstQLine" deals with any header lines, and the "-1" compensates
    for the fact that the first line of the memo is numbered "0".

    wQues will hold the number of the line in the memo with the question
    we want to ask. That will probably NOT be "the question number" as
    perceived by a user.*)

sRecord:=memo1.Lines[wQues];
//showmessage('>>'+sRecord+'<<');
ParseRecord(sRecord,sQID,sHist,sAns,sQ);
laQuestion.caption:=sQ;
//showmessage(sQID+chr($A)+sHist+chr($A)+sAns+chr($A)+sQ);
buPlay.enabled:=false;
eAns.text:='';
eAns.setfocus;


(*

<li>accept an answer from the user,
<li>report if the answer is right or wrong, and
<li>record the user's performance in the memo, and make buSave.enabled true
</ul>*)



end;

procedure TDD74f1.eAnsKeyPress(Sender: TObject; var Key: Char);
var sMassagedAns:string;
    boWasRight:boolean;
begin
//First turn things like 5 into 05, a onto 0a
sMassagedAns:=eAns.text;
if length(sMassagedAns)=1 then sMassagedAns:='0'+sMassagedAns;
//If user has pressed enter key (13) then see if user's answer matches
//       right answer
if Key=chr(13) then begin
  if sMassagedAns=sAns then begin
       laFeedback.caption:='Previous question answered CORRECTLY';
       boWasRight:=true;
       end //no ; here
     else begin
       laFeedback.caption:='Previous question NOT answered correctly';
       boWasRight:=false;
       end;
  //Now write the new datum to the record in the memo...
  sHist:=copy(sHist,1,bHistory-1);
  if boWasRight then sHist:='+'+sHist //no ; here
                else sHist:='x'+sHist;
  memo1.lines[wQues]:=sQID+sHist+sAns+sQ;
  buSave.enabled:=true;
  buPlay.enabled:=true;
  buPlayClick(self);//and run "Play" again.
  end;
end;

procedure TDD74f1.buSaveClick(Sender: TObject);
begin
memo1.Lines.SaveToFile('DD74questions.txt');
buSave.enabled:=false;
end;

procedure TDD74f1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var wTmp:word;
begin
CanClose:=true;
if buSave.enabled=true then begin
  //see accompanying tutorial...
  wTmp:=MessageDlgPos('Performance history has changed.'+chr(10)+
     'Do you want to save the newer data?',
     mtConfirmation, //type of message box
     mbYesNoCancel, //Buttons / return values
     0, //Help context. (Not used here)
     200, 200);  //X/Y of message top left. Screen coords.
  //end "see tutorial"
  if wTmp=mrYes then buSaveClick(self);//Save the data, and leave CanClose true.
  //if wTmp=mrNo then ... do nothing. CanClose will remain true, we close without saving
  if wTmp=mrCancel then CanClose:=false;//This will block attempt to close program
  end;
end;

procedure TDD74f1.buQuitClick(Sender: TObject);
begin
close;(*Use close, not application terminate, if you want
  to do things via the CanClose mechanism.
  Application.Terminate does NOT call CanClose
  {form}.close aborts if CanClose returns False.
  There is no need (in .close) to explicitly call CanClose

  Be careful to ensure that there is a way to get CanClose to
  return "True", or you won't be able to close the application by
  any sensible route.*)

end;

end.

